@@ -28,10 +28,6 @@ class BaseAPI:
     def group(self):
         return self._client.group
 
-    @property
-    def db(self):
-        return self._client.db
-
     def _check_group_id(self):
         if self.group_id == None:
             raise ValueError("group_id is not set yet.")
